function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
import React, { useState, useImperativeHandle, useCallback, useRef, useContext, useMemo, useEffect, useLayoutEffect } from "react";
import { StyleSheet } from "react-native";
import Animated, { useAnimatedStyle, useSharedValue, withSpring, useDerivedValue, useAnimatedReaction, runOnJS, makeMutable } from "react-native-reanimated";
import { Gesture, GestureDetector } from "react-native-gesture-handler";
import { pageInterpolatorCube, pageInterpolatorSlide, pageInterpolatorStack, pageInterpolatorTurnIn } from "./pageInterpolators";
export let Preset = /*#__PURE__*/function (Preset) {
  Preset["SLIDE"] = "slide";
  Preset["CUBE"] = "cube";
  Preset["STACK"] = "stack";
  Preset["TURN_IN"] = "turn-in";
  return Preset;
}({});
const PageInterpolators = {
  [Preset.SLIDE]: pageInterpolatorSlide,
  [Preset.CUBE]: pageInterpolatorCube,
  [Preset.STACK]: pageInterpolatorStack,
  [Preset.TURN_IN]: pageInterpolatorTurnIn
};
export const DEFAULT_ANIMATION_CONFIG = {
  damping: 20,
  mass: 0.2,
  stiffness: 100,
  overshootClamping: false,
  restSpeedThreshold: 0.2,
  restDisplacementThreshold: 0.2
};
const EMPTY_SIMULTANEOUS_GESTURES = [];
const EMPTY_ANIMATION_CONFIG = {};
function InfinitePager(_ref, ref) {
  let {
    vertical = false,
    PageComponent,
    pageCallbackNode,
    onPageChange,
    pageBuffer = 1,
    style,
    pageWrapperStyle,
    minIndex = -Infinity,
    maxIndex = Infinity,
    simultaneousGestures = EMPTY_SIMULTANEOUS_GESTURES,
    gesturesDisabled,
    animationConfig = EMPTY_ANIMATION_CONFIG,
    renderPage,
    flingVelocity = 500,
    preset = Preset.SLIDE,
    pageInterpolator = PageInterpolators[preset],
    bouncePct = 0.0,
    debugTag = "",
    width,
    height,
    minDistance
  } = _ref;
  const orientation = vertical ? "vertical" : "horizontal";
  const pageWidth = useSharedValue(width || 0);
  const pageHeight = useSharedValue(height || 0);
  const pageSize = vertical ? pageHeight : pageWidth;
  const translateX = useSharedValue(0);
  const translateY = useSharedValue(0);
  const translate = vertical ? translateY : translateX;
  const [curIndex, setCurIndex] = useState(0);
  const pageAnimInternal = useSharedValue(0);
  const pageAnim = pageCallbackNode || pageAnimInternal;
  const {
    activePagers,
    nestingDepth,
    pagers
  } = useContext(InfinitePagerContext);
  const parentGestures = useContext(SimultaneousGestureContext);
  const pagerId = useMemo(() => {
    return `${orientation}:${nestingDepth}:${Math.random()}`;
  }, [orientation, nestingDepth]);
  useEffect(() => {
    const updated = new Set(pagers.value);
    updated.add(pagerId);
    pagers.value = [...updated.values()];
    return () => {
      const updated = new Set(pagers.value);
      updated.delete(pagerId);
      pagers.value = [...updated.values()];
    };
  }, [pagerId, pagers]);
  const pageInterpolatorRef = useRef(pageInterpolator);
  pageInterpolatorRef.current = pageInterpolator;
  const curIndexRef = useRef(curIndex);
  curIndexRef.current = curIndex;
  const animCfgRef = useRef(animationConfig);
  animCfgRef.current = animationConfig;
  const gesturesDisabledAnim = useDerivedValue(() => {
    return !!gesturesDisabled;
  }, [gesturesDisabled]);
  const setPage = useCallback(function (index) {
    let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    const updatedTranslate = index * pageSize.value * -1;
    if (index < minIndex || index > maxIndex) return;
    if (options.animated) {
      const animCfg = {
        ...DEFAULT_ANIMATION_CONFIG,
        ...animCfgRef.current
      };
      translate.value = withSpring(updatedTranslate, animCfg);
    } else {
      translate.value = updatedTranslate;
    }
  }, [pageSize, translate, minIndex, maxIndex]);
  useImperativeHandle(ref, () => ({
    setPage,
    incrementPage: options => {
      setPage(curIndexRef.current + 1, options);
    },
    decrementPage: options => {
      setPage(curIndexRef.current - 1, options);
    }
  }), [setPage]);
  const pageIndices = [...Array(pageBuffer * 2 + 1)].map((_, i) => {
    const bufferIndex = i - pageBuffer;
    return curIndex - bufferIndex;
  });
  useDerivedValue(() => {
    if (pageSize.value) {
      pageAnim.value = translate.value / pageSize.value * -1;
    }
  }, [pageSize, pageAnim, translate]);
  function onPageChangeInternal(pg) {
    onPageChange === null || onPageChange === void 0 || onPageChange(pg);
    setCurIndex(pg);
  }
  useAnimatedReaction(() => {
    return Math.round(pageAnim.value);
  }, (cur, prev) => {
    if (cur !== prev) {
      runOnJS(onPageChangeInternal)(cur);
    }
  }, []);
  const startTranslate = useSharedValue(0);
  const panGesture = useMemo(() => Gesture.Pan(), []);
  const minIndexAnim = useDerivedValue(() => {
    return minIndex;
  }, [minIndex]);
  const maxIndexAnim = useDerivedValue(() => {
    return maxIndex;
  }, [maxIndex]);
  const isMinIndex = useDerivedValue(() => {
    return curIndex <= minIndex;
  }, [curIndex, minIndex]);
  const isMaxIndex = useDerivedValue(() => {
    return curIndex >= maxIndex;
  }, [curIndex, maxIndex]);
  const isAtEdge = isMinIndex || isMaxIndex;
  const initTouchX = useSharedValue(0);
  const initTouchY = useSharedValue(0);
  const isGestureLocked = useDerivedValue(() => {
    // Gesture goes to the most-nested active child of both orientations
    // All other pagers are locked
    const isDeepestInOrientation = activePagers.value.filter(v => {
      return v.split(":")[0] === orientation;
    }).every(v => {
      return Number(v.split(":")[1]) <= nestingDepth;
    });
    return activePagers.value.length && !isDeepestInOrientation;
  }, [activePagers, orientation]);
  panGesture.onBegin(evt => {
    "worklet";

    if (!isAtEdge) {
      const updated = activePagers.value.slice();
      updated.push(pagerId);
      activePagers.value = updated;
    }
    startTranslate.value = translate.value;
    initTouchX.value = evt.x;
    initTouchY.value = evt.y;
    if (debugTag) {
      console.log(`${debugTag} onBegin`, evt);
    }
  }).onTouchesMove((evt, mgr) => {
    "worklet";

    const mainTouch = evt.changedTouches[0];
    const evtVal = mainTouch[vertical ? "y" : "x"];
    const initTouch = vertical ? initTouchY.value : initTouchX.value;
    const evtTranslate = evtVal - initTouch;
    const swipingPastEnd = isMinIndex.value && evtTranslate > 0 || isMaxIndex.value && evtTranslate < 0;
    const shouldFailSelf = !bouncePct && swipingPastEnd || isGestureLocked.value || gesturesDisabledAnim.value;
    if (shouldFailSelf) {
      if (debugTag) {
        const failReason = swipingPastEnd ? "range" : "locked";
        const failDetails = swipingPastEnd ? `${isMinIndex.value ? "min" : "max"}, ${evtTranslate}` : "";
        console.log(`${debugTag}: ${failReason} fail (${failDetails})`, evt);
        const updated = activePagers.value.slice().filter(pId => pId !== pagerId);
        activePagers.value = updated;
      }
      mgr.fail();
    } else {
      if (!activePagers.value.includes(pagerId)) {
        const updated = activePagers.value.slice();
        updated.push(pagerId);
        activePagers.value = updated;
      }
    }
  }).onUpdate(evt => {
    "worklet";

    const evtTranslate = vertical ? evt.translationY : evt.translationX;
    const crossAxisTranslate = vertical ? evt.translationX : evt.translationY;
    const isSwipingCrossAxis = Math.abs(crossAxisTranslate) > 10 && Math.abs(crossAxisTranslate) > Math.abs(evtTranslate);
    if (isGestureLocked.value || isSwipingCrossAxis) return;
    if (debugTag) {
      console.log(`${debugTag} onUpdate: ${isGestureLocked.value ? "(locked)" : ""}`, evt);
    }
    const rawVal = startTranslate.value + evtTranslate;
    const page = -rawVal / pageSize.value;
    if (page >= minIndexAnim.value && page <= maxIndexAnim.value) {
      translate.value = rawVal;
    } else {
      const pageTrans = rawVal % pageSize.value;
      const bounceTrans = pageTrans * (1 - bouncePct);
      translate.value = rawVal - bounceTrans;
    }
  }).onEnd(evt => {
    "worklet";

    const evtVelocity = vertical ? evt.velocityY : evt.velocityX;
    const evtTranslate = vertical ? evt.translationY : evt.translationX;
    const crossAxisTranslate = vertical ? evt.translationX : evt.translationY;
    const isSwipingCrossAxis = Math.abs(crossAxisTranslate) > Math.abs(evtTranslate);
    const isFling = isGestureLocked.value || isSwipingCrossAxis ? false : Math.abs(evtVelocity) > flingVelocity;
    let velocityModifier = isFling ? pageSize.value / 2 : 0;
    if (evtVelocity < 0) velocityModifier *= -1;
    let page = -1 * Math.round((translate.value + velocityModifier) / pageSize.value);
    if (page < minIndexAnim.value) page = minIndexAnim.value;
    if (page > maxIndexAnim.value) page = maxIndexAnim.value;
    const animCfg = Object.assign({}, DEFAULT_ANIMATION_CONFIG, animCfgRef.current);
    translate.value = withSpring(-page * pageSize.value, animCfg);
    if (debugTag) {
      console.log(`${debugTag}: onEnd (${isGestureLocked.value ? "locked" : "unlocked"})`, evt);
    }
  }).onFinalize(evt => {
    "worklet";

    const updatedPagerIds = activePagers.value.slice().filter(id => id !== pagerId);
    activePagers.value = updatedPagerIds;
    if (debugTag) {
      console.log(`${debugTag}: onFinalize (${isGestureLocked.value ? "locked" : "unlocked"})`, evt);
    }
  }).enabled(!gesturesDisabled);
  if (typeof minDistance === "number") {
    panGesture.minDistance(minDistance);
  }
  const reInitGesture = useCallback(() => {
    panGesture.initialize();
  }, [panGesture]);
  useLayoutEffect(() => {
    if (nestingDepth === 0) {
      reInitGesture();
    }
  }, [curIndex, minIndex, maxIndex, reInitGesture, pageBuffer, nestingDepth]);
  useAnimatedReaction(() => {
    return pagers.value.join(",");
  }, (val, prev) => {
    if (val && val !== prev && prev !== null && nestingDepth === 0) {
      // For some reason this prevents a bug with nested pagers where, if the outer pager
      // displays a mix of nested and non-nested content,
      // it can become unresponsive when non-nested items enter or exit.
      runOnJS(reInitGesture)();
    }
  }, [pagers, panGesture, nestingDepth]);
  const allGestures = useMemo(() => {
    return [panGesture, ...parentGestures, ...simultaneousGestures];
  }, [panGesture, parentGestures, simultaneousGestures]);
  const wrapperStyle = useMemo(() => {
    const s = {};
    if (width) s.width = width;
    if (height) s.height = height;
    return s;
  }, [width, height]);
  const gesture = useMemo(() => {
    return Gesture.Simultaneous(...allGestures);
  }, [allGestures]);
  return /*#__PURE__*/React.createElement(SimultaneousGestureProvider, {
    simultaneousGestures: allGestures
  }, /*#__PURE__*/React.createElement(GestureDetector, {
    gesture: gesture
  }, /*#__PURE__*/React.createElement(Animated.View, {
    style: [wrapperStyle, style],
    onLayout: _ref2 => {
      let {
        nativeEvent: {
          layout
        }
      } = _ref2;
      pageWidth.value = layout.width;
      pageHeight.value = layout.height;
    }
  }, pageIndices.map(pageIndex => {
    return /*#__PURE__*/React.createElement(PageWrapper, {
      key: `page-provider-wrapper-${pageIndex}`,
      vertical: vertical,
      pageAnim: pageAnim,
      index: pageIndex,
      pageWidth: pageWidth,
      pageHeight: pageHeight,
      isActive: pageIndex === curIndex,
      PageComponent: PageComponent,
      renderPage: renderPage,
      style: pageWrapperStyle,
      pageInterpolatorRef: pageInterpolatorRef,
      pageBuffer: pageBuffer,
      debugTag: debugTag
    });
  }))));
}
const PageWrapper = /*#__PURE__*/React.memo(_ref3 => {
  let {
    index,
    pageAnim,
    pageWidth,
    pageHeight,
    vertical,
    PageComponent,
    renderPage,
    isActive,
    style,
    pageInterpolatorRef,
    pageBuffer
  } = _ref3;
  const pageSize = vertical ? pageHeight : pageWidth;
  const translation = useDerivedValue(() => {
    const translateX = (index - pageAnim.value) * pageSize.value;
    return translateX;
  }, []);
  const focusAnim = useDerivedValue(() => {
    if (!pageSize.value) {
      return index;
    }
    return translation.value / pageSize.value;
  }, []);
  const animStyle = useAnimatedStyle(() => {
    // Short circuit page interpolation to prevent buggy initial values due to possible race condition:
    // https://github.com/software-mansion/react-native-reanimated/issues/2571
    const isInactivePageBeforeInit = index !== 0 && !pageSize.value;
    const _pageWidth = isInactivePageBeforeInit ? focusAnim : pageWidth;
    const _pageHeight = isInactivePageBeforeInit ? focusAnim : pageHeight;
    return pageInterpolatorRef.current({
      focusAnim,
      pageAnim,
      pageWidth: _pageWidth,
      pageHeight: _pageHeight,
      index,
      vertical,
      pageBuffer
    });
  }, [pageWidth, pageHeight, pageAnim, index, translation, vertical, pageBuffer]);
  if (PageComponent && renderPage) {
    console.warn("PageComponent and renderPage both defined, defaulting to PageComponent");
  }
  if (!PageComponent && !renderPage) {
    throw new Error("Either PageComponent or renderPage must be defined.");
  }
  return /*#__PURE__*/React.createElement(Animated.View, {
    pointerEvents: isActive ? "auto" : "none",
    style: [style, styles.pageWrapper, animStyle, isActive && styles.activePage]
  }, PageComponent ? /*#__PURE__*/React.createElement(PageComponent, {
    index: index,
    isActive: isActive,
    focusAnim: focusAnim,
    pageWidthAnim: pageWidth,
    pageHeightAnim: pageHeight,
    pageAnim: pageAnim
  }) : renderPage === null || renderPage === void 0 ? void 0 : renderPage({
    index,
    isActive,
    focusAnim,
    pageWidthAnim: pageWidth,
    pageHeightAnim: pageHeight,
    pageAnim
  }));
});
export default /*#__PURE__*/React.memo(withWrappedProvider( /*#__PURE__*/React.forwardRef(InfinitePager)));
function withWrappedProvider(Inner) {
  return /*#__PURE__*/React.forwardRef((props, ref) => {
    return /*#__PURE__*/React.createElement(InfinitePagerProvider, null, /*#__PURE__*/React.createElement(Inner, _extends({}, props, {
      ref: ref
    })));
  });
}
const styles = StyleSheet.create({
  pageWrapper: {
    left: 0,
    right: 0,
    top: 0,
    bottom: 0,
    position: "absolute"
  },
  activePage: {
    position: "relative"
  }
});
const InfinitePagerContext = /*#__PURE__*/React.createContext({
  activePagers: makeMutable([]),
  pagers: makeMutable([]),
  nestingDepth: -1
});
const SimultaneousGestureContext = /*#__PURE__*/React.createContext([]);
function SimultaneousGestureProvider(_ref4) {
  let {
    simultaneousGestures = EMPTY_SIMULTANEOUS_GESTURES,
    children
  } = _ref4;
  return /*#__PURE__*/React.createElement(SimultaneousGestureContext.Provider, {
    value: simultaneousGestures
  }, children);
}
function InfinitePagerProvider(_ref5) {
  let {
    children
  } = _ref5;
  const {
    nestingDepth,
    activePagers,
    pagers
  } = useContext(InfinitePagerContext);
  const rootPagers = useSharedValue([]);
  const rootActivePagers = useSharedValue([]);
  const value = useMemo(() => {
    const isRoot = nestingDepth === -1;
    return {
      nestingDepth: nestingDepth + 1,
      activePagers: isRoot ? rootActivePagers : activePagers,
      pagers: isRoot ? rootPagers : pagers
    };
  }, [nestingDepth, activePagers, pagers, rootPagers, rootActivePagers]);
  return /*#__PURE__*/React.createElement(InfinitePagerContext.Provider, {
    value: value
  }, children);
}
//# sourceMappingURL=index.js.map